package com.example.restcache;

import javax.inject.Inject;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import org.infinispan.Cache;

/**
 * curl "http://localhost:8080/restcache/rest/asyncstore/put/key01/aaa"
 * curl "http://localhost:8080/restcache/rest/asyncstore/put/key02/bbb"
 * curl "http://localhost:8080/restcache/rest/asyncstore/clear"
 * curl "http://localhost:8080/restcache/rest/asyncstore/get/key01"
 * curl "http://localhost:8080/restcache/rest/asyncstore/get/key02"
 */

@Path("{cache}")
public class CacheOperations {
    @Inject
    CacheManagerService helloService;

    @PathParam("cache")
    private String cacheName;

    private Cache<String, String> getCache() {
        return helloService.cacheManager().getCache(cacheName);
    }

    @GET
    @Path("get/{key}")
    @Produces(MediaType.TEXT_PLAIN)
    public String getFromCache(@PathParam("key") String key) {
        Object value = getCache().get(key);
        return String.format("{key = %s, value = %s}\n", key, value);
    }

    @GET
    @Path("put/{key}/{value}")
    @Produces(MediaType.TEXT_PLAIN)
    public String putToCache(@PathParam("key") String key,
                             @PathParam("value") String value) {
        getCache().put(key, value);
        return String.format("{key = %s, value = %s}\n", key, value);
    }

    @GET
    @Path("remove/{key}")
    @Produces(MediaType.TEXT_PLAIN)
    public String removeFromCache(@PathParam("key") String key) {
        getCache().remove(key);
        return "{removed}\n";
    }

    @GET
    @Path("clear")
    @Produces(MediaType.TEXT_PLAIN)
    public String clearCache() {
        getCache().clear();
        return "{cleared}\n";
    }

    @SuppressWarnings("deprecation")
    @GET
    @Path("clearasync")
    @Produces(MediaType.TEXT_PLAIN)
    public String clearAsyncCache() {
        try {
            getCache().clearAsync().get();
            return "{async cleared}\n";
        } catch (Exception e) {
            e.printStackTrace();
            return "{async clear failed}\n";
        }
    }
}
